@@ -1,22 +1,24 @@
 """Scaling-related mathematical functions.
 """
 
+import numpy as np
+
 import learnergy.utils.constants as c
 
 
-def unitary_scale(x):
+def unitary_scale(x: np.array) -> np.array:
     """Scales an array between 0 and 1.
 
     Args:
-        x (array): A numpy array to be scaled.
+        x: A numpy array to be scaled.
 
     Returns:
-        The scaled array.
+        (np.array): Scaled array.
 
     """
 
     # Makes sure the array is float typed
-    x = x.astype('float32')
+    x = x.astype("float32")
 
     # Gathers array minimum and subtract
     x -= x.min()
