@@ -2,6 +2,7 @@
 """
 
 import time
+from typing import Optional, Tuple
 
 import torch
 import torch.nn as nn
@@ -10,10 +11,10 @@ from torch.utils.data import DataLoader
 from tqdm import tqdm
 
 import learnergy.utils.exception as e
-import learnergy.utils.logging as l
 from learnergy.models.bernoulli import RBM
+from learnergy.utils import logging
 
-logger = l.get_logger(__name__)
+logger = logging.get_logger(__name__)
 
 
 class DiscriminativeRBM(RBM):
@@ -26,28 +27,45 @@ class DiscriminativeRBM(RBM):
 
     """
 
-    def __init__(self, n_visible=128, n_hidden=128, n_classes=1, steps=1,
-                 learning_rate=0.1, momentum=0, decay=0, temperature=1, use_gpu=False):
+    def __init__(
+        self,
+        n_visible: Optional[int] = 128,
+        n_hidden: Optional[int] = 128,
+        n_classes: Optional[int] = 1,
+        steps: Optional[int] = 1,
+        learning_rate: Optional[float] = 0.1,
+        momentum: Optional[float] = 0.0,
+        decay: Optional[float] = 0.0,
+        temperature: Optional[float] = 1.0,
+        use_gpu: Optional[bool] = False,
+    ) -> None:
         """Initialization method.
 
         Args:
-            n_visible (int): Amount of visible units.
-            n_hidden (int): Amount of hidden units.
-            n_classes (int): Amount of classes.
-            steps (int): Number of Gibbs' sampling steps.
-            learning_rate (float): Learning rate.
-            momentum (float): Momentum parameter.
-            decay (float): Weight decay used for penalization.
-            temperature (float): Temperature factor.
-            use_gpu (boolean): Whether GPU should be used or not.
+            n_visible: Amount of visible units.
+            n_hidden: Amount of hidden units.
+            n_classes: Amount of classes.
+            steps: Number of Gibbs' sampling steps.
+            learning_rate: Learning rate.
+            momentum: Momentum parameter.
+            decay: Weight decay used for penalization.
+            temperature: Temperature factor.
+            use_gpu: Whether GPU should be used or not.
 
         """
 
-        logger.info('Overriding class: RBM -> DiscriminativeRBM.')
+        logger.info("Overriding class: RBM -> DiscriminativeRBM.")
 
-        # Override its parent class
-        super(DiscriminativeRBM, self).__init__(n_visible, n_hidden, steps, learning_rate,
-                                                momentum, decay, temperature, use_gpu)
+        super(DiscriminativeRBM, self).__init__(
+            n_visible,
+            n_hidden,
+            steps,
+            learning_rate,
+            momentum,
+            decay,
+            temperature,
+            use_gpu,
+        )
 
         # Number of classes
         self.n_classes = n_classes
@@ -62,94 +80,74 @@ class DiscriminativeRBM(RBM):
         self.loss = nn.CrossEntropyLoss()
 
         # Updating optimizer's parameters with `U`
-        self.optimizer.add_param_group({'params': self.U})
+        self.optimizer.add_param_group({"params": self.U})
 
         # Updating optimizer's parameters with `c`
-        self.optimizer.add_param_group({'params': self.c})
+        self.optimizer.add_param_group({"params": self.c})
 
         # Re-checks if current device is CUDA-based due to new parameter
-        if self.device == 'cuda':
+        if self.device == "cuda":
             # If yes, re-uses CUDA in the whole class
             self.cuda()
 
-        logger.info('Class overrided.')
+        logger.info("Class overrided.")
 
     @property
-    def n_classes(self):
-        """int: Number of classes.
-
-        """
+    def n_classes(self) -> int:
+        """Number of classes."""
 
         return self._n_classes
 
     @n_classes.setter
-    def n_classes(self, n_classes):
-        if not isinstance(n_classes, int):
-            raise e.TypeError('`n_classes` should be an integer')
+    def n_classes(self, n_classes: int) -> None:
         if n_classes <= 0:
-            raise e.ValueError('`n_classes` should be > 0')
+            raise e.ValueError("`n_classes` should be > 0")
 
         self._n_classes = n_classes
 
     @property
-    def U(self):
-        """torch.nn.Parameter: Class weights' matrix.
-
-        """
+    def U(self) -> torch.nn.Parameter:
+        """Class weights' matrix."""
 
         return self._U
 
     @U.setter
-    def U(self, U):
-        if not isinstance(U, nn.Parameter):
-            raise e.TypeError('`U` should be a PyTorch parameter')
-
+    def U(self, U: torch.nn.Parameter) -> None:
         self._U = U
 
     @property
-    def c(self):
-        """torch.nn.Parameter: Class units bias.
-
-        """
+    def c(self) -> torch.nn.Parameter:
+        """Class units bias."""
 
         return self._c
 
     @c.setter
-    def c(self, c):
-        if not isinstance(c, nn.Parameter):
-            raise e.TypeError('`c` should be a PyTorch parameter')
-
+    def c(self, c: torch.nn.Parameter) -> None:
         self._c = c
 
     @property
-    def loss(self):
-        """torch.nn.CrossEntropyLoss: Cross-Entropy loss function.
-
-        """
+    def loss(self) -> torch.nn.CrossEntropyLoss:
+        """Cross-Entropy loss function."""
 
         return self._loss
 
     @loss.setter
-    def loss(self, loss):
-        if not isinstance(loss, nn.CrossEntropyLoss):
-            raise e.TypeError('`loss` should be a CrossEntropy')
-
+    def loss(self, loss: torch.nn.CrossEntropyLoss) -> None:
         self._loss = loss
 
-    def labels_sampling(self, samples):
+    def labels_sampling(self, samples: torch.Tensor) -> torch.Tensor:
         """Calculates labels probabilities by samplings, i.e., P(y|v).
 
         Args:
-            samples (torch.Tensor): Samples to be labels-calculated.
+            samples: Samples to be labels-calculated.
 
         Returns:
-            Labels' probabilities based on input samples.
+            (torch.Tensor): Labels' probabilities based on input samples.
 
         """
 
         # Creating an empty tensor for holding the probabilities per class
-        probs = torch.zeros(samples.size(
-            0), self.n_classes, device=self.device)
+        probs = torch.zeros(samples.size(0), self.n_classes, device=self.device)
 
         # Calculate samples' activations
         activations = F.linear(samples, self.W.t(), self.b)
@@ -160,34 +158,39 @@ class DiscriminativeRBM(RBM):
         # Iterating through every possible class
         for i in range(self.n_classes):
             # Calculates the logit-probability for the particular class
-            probs[:, i] = self.c[i] + \
-                torch.sum(s(activations + self.U[i, :]), dim=1)
+            probs[:, i] = self.c[i] + torch.sum(s(activations + self.U[i, :]), dim=1)
 
         # Recovering the predictions based on the logit-probabilities
         preds = torch.argmax(probs.detach(), 1)
 
         return probs, preds
 
-    def fit(self, dataset, batch_size=128, epochs=10):
+    def fit(
+        self,
+        dataset: torch.utils.data.Dataset,
+        batch_size: Optional[int] = 128,
+        epochs: Optional[int] = 10,
+    ) -> Tuple[float, float]:
         """Fits a new DRBM model.
 
         Args:
-            dataset (torch.utils.data.Dataset): A Dataset object containing the training data.
-            batch_size (int): Amount of samples per batch.
-            epochs (int): Number of training epochs.
+            dataset: A Dataset object containing the training data.
+            batch_size: Amount of samples per batch.
+            epochs: Number of training epochs.
 
         Returns:
-            Loss and accuracy from the training step.
+            (Tuple[float, float]): Loss and accuracy from the training step.
 
         """
 
         # Transforming the dataset into training batches
-        batches = DataLoader(dataset, batch_size=batch_size,
-                             shuffle=True, num_workers=0)
+        batches = DataLoader(
+            dataset, batch_size=batch_size, shuffle=True, num_workers=0
+        )
 
         # For every epoch
         for epoch in range(epochs):
-            logger.info('Epoch %d/%d', epoch+1, epochs)
+            logger.info("Epoch %d/%d", epoch + 1, epochs)
 
             # Calculating the time of the epoch's starting
             start = time.time()
@@ -202,7 +205,7 @@ class DiscriminativeRBM(RBM):
                 samples = samples.reshape(len(samples), self.n_visible)
 
                 # Checking whether GPU is avaliable and if it should be used
-                if self.device == 'cuda':
+                if self.device == "cuda":
                     # Applies the GPU usage to the data and labels
                     samples = samples.cuda()
                     labels = labels.cuda()
@@ -230,7 +233,8 @@ class DiscriminativeRBM(RBM):
 
                 # Calculating current's batch accuracy
                 batch_acc = torch.mean(
-                    (torch.sum(preds == labels).float()) / batch_size)
+                    (torch.sum(preds == labels).float()) / batch_size
+                )
 
                 # Summing up to epochs' loss and accuracy
                 loss += cost.detach()
@@ -244,24 +248,26 @@ class DiscriminativeRBM(RBM):
             end = time.time()
 
             # Dumps the desired variables to the model's history
-            self.dump(loss=loss.item(), acc=acc.item(), time=end-start)
+            self.dump(loss=loss.item(), acc=acc.item(), time=end - start)
 
-            logger.info('Loss: %f | Accuracy: %f', loss, acc)
+            logger.info("Loss: %f | Accuracy: %f", loss, acc)
 
         return loss, acc
 
-    def predict(self, dataset):
+    def predict(
+        self, dataset: torch.utils.data.Dataset
+    ) -> Tuple[float, torch.Tensor, torch.Tensor]:
         """Predicts batches of new samples.
 
         Args:
-            dataset (torch.utils.data.Dataset): A Dataset object containing the testing data.
+            dataset: A Dataset object containing the testing data.
 
         Returns:
-            Prediction probabilities and labels, i.e., P(y|v).
+            (Tuple[float, torch.Tensor, torch.Tensor]): Accuracy, prediction probabilities and labels, i.e., P(y|v).
 
         """
 
-        logger.info('Predicting new samples ...')
+        logger.info("Predicting new samples ...")
 
         # Resetting accuracy to zero
         acc = 0
@@ -270,8 +276,9 @@ class DiscriminativeRBM(RBM):
         batch_size = len(dataset)
 
         # Transforming the dataset into training batches
-        batches = DataLoader(dataset, batch_size=batch_size,
-                             shuffle=False, num_workers=0)
+        batches = DataLoader(
+            dataset, batch_size=batch_size, shuffle=False, num_workers=0
+        )
 
         # For every batch
         for samples, labels in tqdm(batches):
@@ -279,7 +286,7 @@ class DiscriminativeRBM(RBM):
             samples = samples.reshape(len(samples), self.n_visible)
 
             # Checking whether GPU is avaliable and if it should be used
-            if self.device == 'cuda':
+            if self.device == "cuda":
                 # Applies the GPU usage to the data and labels
                 samples = samples.cuda()
                 labels = labels.cuda()
@@ -288,8 +295,7 @@ class DiscriminativeRBM(RBM):
             probs, preds = self.labels_sampling(samples)
 
             # Calculating current's batch accuracy
-            batch_acc = torch.mean(
-                (torch.sum(preds == labels).float()) / batch_size)
+            batch_acc = torch.mean((torch.sum(preds == labels).float()) / batch_size)
 
             # Summing up the prediction accuracy
             acc += batch_acc
@@ -297,7 +303,7 @@ class DiscriminativeRBM(RBM):
         # Normalizing the accuracy with the number of batches
         acc /= len(batches)
 
-        logger.info('Accuracy: %f', acc)
+        logger.info("Accuracy: %f", acc)
 
         return acc, probs, preds
 
@@ -312,62 +318,78 @@ class HybridDiscriminativeRBM(DiscriminativeRBM):
 
     """
 
-    def __init__(self, n_visible=128, n_hidden=128, n_classes=1, steps=1, learning_rate=0.1,
-                 alpha=0.01, momentum=0, decay=0, temperature=1, use_gpu=False):
+    def __init__(
+        self,
+        n_visible: Optional[int] = 128,
+        n_hidden: Optional[int] = 128,
+        n_classes: Optional[int] = 1,
+        steps: Optional[int] = 1,
+        learning_rate: Optional[float] = 0.1,
+        alpha: Optional[float] = 0.01,
+        momentum: Optional[float] = 0.0,
+        decay: Optional[float] = 0.0,
+        temperature: Optional[float] = 1.0,
+        use_gpu: Optional[bool] = False,
+    ) -> None:
         """Initialization method.
 
         Args:
-            n_visible (int): Amount of visible units.
-            n_hidden (int): Amount of hidden units.
-            n_classes (int): Amount of classes.
-            steps (int): Number of Gibbs' sampling steps.
-            learning_rate (float): Learning rate.
-            alpha (float): Amount of penalization to the generative loss.
-            momentum (float): Momentum parameter.
-            decay (float): Weight decay used for penalization.
-            temperature (float): Temperature factor.
-            use_gpu (boolean): Whether GPU should be used or not.
+            n_visible: Amount of visible units.
+            n_hidden: Amount of hidden units.
+            n_classes: Amount of classes.
+            steps: Number of Gibbs' sampling steps.
+            learning_rate: Learning rate.
+            alpha: Amount of penalization to the generative loss.
+            momentum: Momentum parameter.
+            decay: Weight decay used for penalization.
+            temperature: Temperature factor.
+            use_gpu: Whether GPU should be used or not.
 
         """
 
-        logger.info(
-            'Overriding class: DiscriminativeRBM -> HybridDiscriminativeRBM.')
+        logger.info("Overriding class: DiscriminativeRBM -> HybridDiscriminativeRBM.")
 
         # Override its parent class
-        super(HybridDiscriminativeRBM, self).__init__(n_visible, n_hidden, n_classes, steps,
-                                                      learning_rate, momentum, decay, temperature,
-                                                      use_gpu)
+        super(HybridDiscriminativeRBM, self).__init__(
+            n_visible,
+            n_hidden,
+            n_classes,
+            steps,
+            learning_rate,
+            momentum,
+            decay,
+            temperature,
+            use_gpu,
+        )
 
         # Defining a property for the generative loss penalization
         self.alpha = alpha
 
     @property
-    def alpha(self):
-        """float: Generative loss penalization.
-
-        """
+    def alpha(self) -> float:
+        """Generative loss penalization."""
 
         return self._alpha
 
     @alpha.setter
-    def alpha(self, alpha):
-        if not isinstance(alpha, (float, int)):
-            raise e.TypeError('`alpha` should be a float or integer')
+    def alpha(self, alpha: float) -> None:
         if alpha < 0:
-            raise e.ValueError('`alpha` should be >= 0')
+            raise e.ValueError("`alpha` should be >= 0")
 
         self._alpha = alpha
 
-    def hidden_sampling(self, v, y, scale=False):
+    def hidden_sampling(
+        self, v: torch.Tensor, y: torch.Tensor, scale: Optional[bool] = False
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
         """Performs the hidden layer sampling, i.e., P(h|y,v).
 
         Args:
-            v (torch.Tensor): A tensor incoming from the visible layer.
-            y (torch.Tensor): A tensor incoming from the class layer.
-            scale (bool): A boolean to decide whether temperature should be used or not.
+            v: A tensor incoming from the visible layer.
+            y: A tensor incoming from the class layer.
+            scale: A boolean to decide whether temperature should be used or not.
 
         Returns:
-            The probabilities and states of the hidden layer sampling.
+            (Tuple[torch.Tensor, torch.Tensor]): The probabilities and states of the hidden layer sampling.
 
         """
 
@@ -389,42 +411,43 @@ class HybridDiscriminativeRBM(DiscriminativeRBM):
 
         return probs, states
 
-    def class_sampling(self, h):
+    def class_sampling(self, h: torch.Tensor) -> Tuple[torch.Tensor, torch.Tensor]:
         """Performs the class layer sampling, i.e., P(y|h).
 
         Args:
-            h (torch.Tensor): A tensor incoming from the hidden layer.
+            h: A tensor incoming from the hidden layer.
 
         Returns:
-            The probabilities and states of the class layer sampling.
+            (Tuple[torch.Tensor, torch.Tensor]): The probabilities and states of the class layer sampling.
 
         """
 
         # Calculating neurons' activations
         activations = torch.exp(F.linear(h, self.U, self.c))
 
-        # Normalizing activations to calculate probabilities
-        # probs = torch.nn.functional.normalize(activations, p=1, dim=1)
-        probs = torch.div(activations, torch.sum(
-            activations, dim=1).unsqueeze(1))
+        # Calculating activations
+        probs = torch.div(activations, torch.sum(activations, dim=1).unsqueeze(1))
 
         # Sampling current states
-        states = torch.nn.functional.one_hot(torch.argmax(
-            probs, dim=1), num_classes=self.n_classes).float()
+        states = torch.nn.functional.one_hot(
+            torch.argmax(probs, dim=1), num_classes=self.n_classes
+        ).float()
 
         return probs, states
 
-    def gibbs_sampling(self, v, y):
+    def gibbs_sampling(
+        self, v: torch.Tensor, y: torch.Tensor
+    ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor]:
         """Performs the whole Gibbs sampling procedure.
 
         Args:
-            v (torch.Tensor): A tensor incoming from the visible layer.
-            y (torch.Tensor): A tensor incoming from the class layer.
+            v: A tensor incoming from the visible layer.
+            y: A tensor incoming from the class layer.
 
         Returns:
-            The probabilities and states of the hidden layer sampling (positive),
-            the probabilities and states of the hidden layer sampling (negative)
-            and the states of the visible layer sampling (negative).
+            (Tuple[torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor]): The probabilities and states of the hidden layer sampling (positive),
+                the probabilities and states of the hidden layer sampling (negative)
+                and the states of the visible layer sampling (negative).
 
         """
 
@@ -440,45 +463,56 @@ class HybridDiscriminativeRBM(DiscriminativeRBM):
         # Performing the Contrastive Divergence
         for _ in range(self.steps):
             # Calculating visible probabilities and states
-            _, visible_states = self.visible_sampling(
-                neg_hidden_states, True)
+            _, visible_states = self.visible_sampling(neg_hidden_states, True)
 
             # Calculating class probabilities and states
             _, class_states = self.class_sampling(neg_hidden_states)
 
             # Calculating hidden probabilities and states
             neg_hidden_probs, neg_hidden_states = self.hidden_sampling(
-                visible_states, class_states, True)
+                visible_states, class_states, True
+            )
 
-        return pos_hidden_probs, pos_hidden_states, neg_hidden_probs, neg_hidden_states, visible_states
+        return (
+            pos_hidden_probs,
+            pos_hidden_states,
+            neg_hidden_probs,
+            neg_hidden_states,
+            visible_states,
+        )
 
-    def fit(self, dataset, batch_size=128, epochs=10):
+    def fit(
+        self,
+        dataset: torch.utils.data.Dataset,
+        batch_size: Optional[int] = 128,
+        epochs: Optional[int] = 10,
+    ) -> Tuple[float, float]:
         """Fits a new DRBM model.
 
         Args:
-            dataset (torch.utils.data.Dataset): A Dataset object containing the training data.
-            batch_size (int): Amount of samples per batch.
-            epochs (int): Number of training epochs.
+            dataset: A Dataset object containing the training data.
+            batch_size: Amount of samples per batch.
+            epochs: Number of training epochs.
 
         Returns:
-            Loss and accuracy from the training step.
+            (Tuple[float, float]): Loss and accuracy from the training step.
 
         """
 
         # Transforming the dataset into training batches
-        batches = DataLoader(dataset, batch_size=batch_size,
-                             shuffle=True, num_workers=0)
+        batches = DataLoader(
+            dataset, batch_size=batch_size, shuffle=True, num_workers=0
+        )
 
         # For every epoch
         for epoch in range(epochs):
-            logger.info('Epoch %d/%d', epoch+1, epochs)
+            logger.info("Epoch %d/%d", epoch + 1, epochs)
 
             # Calculating the time of the epoch's starting
             start = time.time()
 
             # Resetting epoch's losses and accuracy to zero
-            d_loss, g_loss, loss = 0, 0, 0
-            acc = 0
+            d_loss, g_loss, loss, acc = 0, 0, 0, 0
 
             # For every batch
             for samples, labels in tqdm(batches):
@@ -486,14 +520,13 @@ class HybridDiscriminativeRBM(DiscriminativeRBM):
                 samples = samples.reshape(len(samples), self.n_visible)
 
                 # Checking whether GPU is avaliable and if it should be used
-                if self.device == 'cuda':
+                if self.device == "cuda":
                     # Applies the GPU usage to the data and labels
                     samples = samples.cuda()
                     labels = labels.cuda()
 
                 # Performs the Gibbs sampling procedure
-                _, _, _, _, visible_states = self.gibbs_sampling(
-                    samples, labels)
+                _, _, _, _, visible_states = self.gibbs_sampling(samples, labels)
 
                 # Detaching the visible states from GPU for further computation
                 visible_states = visible_states.detach()
@@ -527,7 +560,8 @@ class HybridDiscriminativeRBM(DiscriminativeRBM):
 
                 # Calculating current's batch accuracy
                 batch_acc = torch.mean(
-                    (torch.sum(preds == labels).float()) / batch_size)
+                    (torch.sum(preds == labels).float()) / batch_size
+                )
 
                 # Summing up to epochs' genator, discriminator and total loss, and accuracy
                 d_loss += d_cost
@@ -545,10 +579,20 @@ class HybridDiscriminativeRBM(DiscriminativeRBM):
             end = time.time()
 
             # Dumps the desired variables to the model's history
-            self.dump(d_loss=d_loss.item(), g_loss=g_loss.item(), loss=loss.item(),
-                      acc=acc.item(), time=end-start)
+            self.dump(
+                d_loss=d_loss.item(),
+                g_loss=g_loss.item(),
+                loss=loss.item(),
+                acc=acc.item(),
+                time=end - start,
+            )
 
             logger.info(
-                'Loss(D): %f | Loss(G): %f | Loss: %f | Accuracy: %f', d_loss, g_loss, loss, acc)
+                "Loss(D): %f | Loss(G): %f | Loss: %f | Accuracy: %f",
+                d_loss,
+                g_loss,
+                loss,
+                acc,
+            )
 
         return loss, acc
