@@ -2,6 +2,7 @@
 """
 
 import time
+from typing import Optional, Tuple
 
 import torch
 import torch.nn as nn
@@ -12,10 +13,10 @@ from tqdm import tqdm
 
 import learnergy.utils.constants as c
 import learnergy.utils.exception as e
-import learnergy.utils.logging as l
 from learnergy.core import Model
+from learnergy.utils import logging
 
-logger = l.get_logger(__name__)
+logger = logging.get_logger(__name__)
 
 
 class RBM(Model):
@@ -27,25 +28,33 @@ class RBM(Model):
 
     """
 
-    def __init__(self, n_visible=128, n_hidden=128, steps=1,
-                 learning_rate=0.1, momentum=0, decay=0, temperature=1, use_gpu=False):
+    def __init__(
+        self,
+        n_visible: Optional[int] = 128,
+        n_hidden: Optional[int] = 128,
+        steps: Optional[int] = 1,
+        learning_rate: Optional[float] = 0.1,
+        momentum: Optional[float] = 0.0,
+        decay: Optional[float] = 0.0,
+        temperature: Optional[float] = 1.0,
+        use_gpu: Optional[bool] = False,
+    ) -> None:
         """Initialization method.
 
         Args:
-            n_visible (int): Amount of visible units.
-            n_hidden (int): Amount of hidden units.
-            steps (int): Number of Gibbs' sampling steps.
-            learning_rate (float): Learning rate.
-            momentum (float): Momentum parameter.
-            decay (float): Weight decay used for penalization.
-            temperature (float): Temperature factor.
-            use_gpu (boolean): Whether GPU should be used or not.
+            n_visible: Amount of visible units.
+            n_hidden: Amount of hidden units.
+            steps: Number of Gibbs' sampling steps.
+            learning_rate: Learning rate.
+            momentum: Momentum parameter.
+            decay: Weight decay used for penalization.
+            temperature: Temperature factor.
+            use_gpu: Whether GPU should be used or not.
 
         """
 
-        logger.info('Overriding class: Model -> RBM.')
+        logger.info("Overriding class: Model -> RBM.")
 
-        # Override its parent class
         super(RBM, self).__init__(use_gpu=use_gpu)
 
         # Amount of visible units
@@ -80,206 +89,169 @@ class RBM(Model):
 
         # Creating the optimizer object
         self.optimizer = opt.SGD(
-            self.parameters(), lr=learning_rate, momentum=momentum, weight_decay=decay)
+            self.parameters(), lr=learning_rate, momentum=momentum, weight_decay=decay
+        )
 
         # Checks if current device is CUDA-based
-        if self.device == 'cuda':
+        if self.device == "cuda":
             # If yes, uses CUDA in the whole class
             self.cuda()
 
-        logger.info('Class overrided.')
-        logger.debug('Size: (%d, %d) | Learning: CD-%d | '
-                     'Hyperparameters: lr = %s, momentum = %s, decay = %s, T = %s.',
-                     self.n_visible, self.n_hidden, self.steps, self.lr, self.momentum, self.decay, self.T)
+        logger.info("Class overrided.")
+        logger.debug(
+            "Size: (%d, %d) | Learning: CD-%d | "
+            "Hyperparameters: lr = %s, momentum = %s, decay = %s, T = %s.",
+            self.n_visible,
+            self.n_hidden,
+            self.steps,
+            self.lr,
+            self.momentum,
+            self.decay,
+            self.T,
+        )
 
     @property
-    def n_visible(self):
-        """int: Number of visible units.
-
-        """
+    def n_visible(self) -> int:
+        """Number of visible units."""
 
         return self._n_visible
 
     @n_visible.setter
-    def n_visible(self, n_visible):
-        if not isinstance(n_visible, int):
-            raise e.TypeError('`n_visible` should be an integer')
+    def n_visible(self, n_visible: int) -> None:
         if n_visible <= 0:
-            raise e.ValueError('`n_visible` should be > 0')
+            raise e.ValueError("`n_visible` should be > 0")
 
         self._n_visible = n_visible
 
     @property
-    def n_hidden(self):
-        """int: Number of hidden units.
-
-        """
+    def n_hidden(self) -> int:
+        """Number of hidden units."""
 
         return self._n_hidden
 
     @n_hidden.setter
-    def n_hidden(self, n_hidden):
-        if not isinstance(n_hidden, int):
-            raise e.TypeError('`n_hidden` should be an integer')
+    def n_hidden(self, n_hidden: int) -> None:
         if n_hidden <= 0:
-            raise e.ValueError('`n_hidden` should be > 0')
+            raise e.ValueError("`n_hidden` should be > 0")
 
         self._n_hidden = n_hidden
 
     @property
-    def steps(self):
-        """int: Number of steps Gibbs' sampling steps.
-
-        """
+    def steps(self) -> int:
+        """Number of steps Gibbs' sampling steps."""
 
         return self._steps
 
     @steps.setter
-    def steps(self, steps):
-        if not isinstance(steps, int):
-            raise e.TypeError('`steps` should be an integer')
+    def steps(self, steps: int) -> None:
         if steps <= 0:
-            raise e.ValueError('`steps` should be > 0')
+            raise e.ValueError("`steps` should be > 0")
 
         self._steps = steps
 
     @property
-    def lr(self):
-        """float: Learning rate.
-
-        """
+    def lr(self) -> float:
+        """Learning rate."""
 
         return self._lr
 
     @lr.setter
-    def lr(self, lr):
-        if not isinstance(lr, (float, int)):
-            raise e.TypeError('`lr` should be a float or integer')
+    def lr(self, lr: float) -> None:
         if lr < 0:
-            raise e.ValueError('`lr` should be >= 0')
+            raise e.ValueError("`lr` should be >= 0")
 
         self._lr = lr
 
     @property
-    def momentum(self):
-        """float: Momentum parameter.
-
-        """
+    def momentum(self) -> float:
+        """Momentum parameter."""
 
         return self._momentum
 
     @momentum.setter
-    def momentum(self, momentum):
-        if not isinstance(momentum, (float, int)):
-            raise e.TypeError('`momentum` should be a float or integer')
+    def momentum(self, momentum: float) -> None:
         if momentum < 0:
-            raise e.ValueError('`momentum` should be >= 0')
+            raise e.ValueError("`momentum` should be >= 0")
 
         self._momentum = momentum
 
     @property
-    def decay(self):
-        """float: Weight decay.
-
-        """
+    def decay(self) -> float:
+        """Weight decay."""
 
         return self._decay
 
     @decay.setter
-    def decay(self, decay):
-        if not isinstance(decay, (float, int)):
-            raise e.TypeError('`decay` should be a float or integer')
+    def decay(self, decay: float) -> None:
         if decay < 0:
-            raise e.ValueError('`decay` should be >= 0')
+            raise e.ValueError("`decay` should be >= 0")
 
         self._decay = decay
 
     @property
-    def T(self):
-        """float: Temperature factor.
-
-        """
+    def T(self) -> float:
+        """Temperature factor."""
 
         return self._T
 
     @T.setter
-    def T(self, T):
-        if not isinstance(T, (float, int)):
-            raise e.TypeError('`T` should be a float or integer')
-        if T < 0 or T > 1:
-            raise e.ValueError('`T` should be between 0 and 1')
+    def T(self, T: float) -> None:
+        if T <= 0 or T > 1:
+            raise e.ValueError("`T` should be between 0 and 1")
 
         self._T = T
 
     @property
-    def W(self):
-        """torch.nn.Parameter: Weights' matrix.
-
-        """
+    def W(self) -> torch.nn.Parameter:
+        """Weights' matrix."""
 
         return self._W
 
     @W.setter
-    def W(self, W):
-        if not isinstance(W, nn.Parameter):
-            raise e.TypeError('`W` should be a PyTorch parameter')
-
+    def W(self, W: torch.nn.Parameter) -> None:
         self._W = W
 
     @property
-    def a(self):
-        """torch.nn.Parameter: Visible units bias.
-
-        """
+    def a(self) -> torch.nn.Parameter:
+        """Visible units bias."""
 
         return self._a
 
     @a.setter
-    def a(self, a):
-        if not isinstance(a, nn.Parameter):
-            raise e.TypeError('`a` should be a PyTorch parameter')
-
+    def a(self, a: torch.nn.Parameter) -> None:
         self._a = a
 
     @property
-    def b(self):
-        """torch.nn.Parameter: Hidden units bias.
-
-        """
+    def b(self) -> torch.nn.Parameter:
+        """Hidden units bias."""
 
         return self._b
 
     @b.setter
-    def b(self, b):
-        if not isinstance(b, nn.Parameter):
-            raise e.TypeError('`b` should be a PyTorch parameter')
-
+    def b(self, b: torch.nn.Parameter) -> None:
         self._b = b
 
     @property
-    def optimizer(self):
-        """torch.optim.SGD: Stochastic Gradient Descent object.
-
-        """
+    def optimizer(self) -> torch.optim.SGD:
+        """Stochastic Gradient Descent object."""
 
         return self._optimizer
 
     @optimizer.setter
-    def optimizer(self, optimizer):
-        if not isinstance(optimizer, opt.SGD):
-            raise e.TypeError('`optimizer` should be a SGD')
-
+    def optimizer(self, optimizer: torch.optim.SGD) -> None:
         self._optimizer = optimizer
 
-    def pre_activation(self, v, scale=False):
+    def pre_activation(
+        self, v: torch.Tensor, scale: Optional[bool] = False
+    ) -> torch.Tensor:
         """Performs the pre-activation over hidden neurons, i.e., Wx' + b.
 
         Args:
-            v (torch.Tensor): A tensor incoming from the visible layer.
-            scale (bool): A boolean to decide whether temperature should be used or not.
+            v: A tensor incoming from the visible layer.
+            scale: A boolean to decide whether temperature should be used or not.
 
         Returns:
-            An input for any type of activation function.
+            (torch.Tensor): An input for any type of activation function.
 
         """
 
@@ -293,15 +265,17 @@ class RBM(Model):
 
         return activations
 
-    def hidden_sampling(self, v, scale=False):
+    def hidden_sampling(
+        self, v: torch.Tensor, scale: Optional[bool] = False
+    ) -> torch.Tensor:
         """Performs the hidden layer sampling, i.e., P(h|v).
 
         Args:
-            v (torch.Tensor): A tensor incoming from the visible layer.
-            scale (bool): A boolean to decide whether temperature should be used or not.
+            v: A tensor incoming from the visible layer.
+            scale: A boolean to decide whether temperature should be used or not.
 
         Returns:
-            The probabilities and states of the hidden layer sampling.
+            (torch.Tensor): The probabilities and states of the hidden layer sampling.
 
         """
 
@@ -323,15 +297,17 @@ class RBM(Model):
 
         return probs, states
 
-    def visible_sampling(self, h, scale=False):
+    def visible_sampling(
+        self, h: torch.Tensor, scale: Optional[bool] = False
+    ) -> torch.Tensor:
         """Performs the visible layer sampling, i.e., P(v|h).
 
         Args:
-            h (torch.Tensor): A tensor incoming from the hidden layer.
-            scale (bool): A boolean to decide whether temperature should be used or not.
+            h: A tensor incoming from the hidden layer.
+            scale: A boolean to decide whether temperature should be used or not.
 
         Returns:
-            The probabilities and states of the visible layer sampling.
+            (torch.Tensor): The probabilities and states of the visible layer sampling.
 
         """
 
@@ -353,16 +329,18 @@ class RBM(Model):
 
         return probs, states
 
-    def gibbs_sampling(self, v):
+    def gibbs_sampling(
+        self, v: torch.Tensor
+    ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor]:
         """Performs the whole Gibbs sampling procedure.
 
         Args:
-            v (torch.Tensor): A tensor incoming from the visible layer.
+            v: A tensor incoming from the visible layer.
 
         Returns:
-            The probabilities and states of the hidden layer sampling (positive),
-            the probabilities and states of the hidden layer sampling (negative)
-            and the states of the visible layer sampling (negative).
+            (Tuple[torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor]): The probabilities and states of the hidden layer sampling (positive),
+                the probabilities and states of the hidden layer sampling (negative)
+                and the states of the visible layer sampling (negative).
 
         """
 
@@ -375,23 +353,29 @@ class RBM(Model):
         # Performing the Contrastive Divergence
         for _ in range(self.steps):
             # Calculating visible probabilities and states
-            _, visible_states = self.visible_sampling(
-                neg_hidden_states, True)
+            _, visible_states = self.visible_sampling(neg_hidden_states, True)
 
             # Calculating hidden probabilities and states
             neg_hidden_probs, neg_hidden_states = self.hidden_sampling(
-                visible_states, True)
+                visible_states, True
+            )
 
-        return pos_hidden_probs, pos_hidden_states, neg_hidden_probs, neg_hidden_states, visible_states
+        return (
+            pos_hidden_probs,
+            pos_hidden_states,
+            neg_hidden_probs,
+            neg_hidden_states,
+            visible_states,
+        )
 
-    def energy(self, samples):
+    def energy(self, samples: torch.Tensor) -> torch.Tensor:
         """Calculates and frees the system's energy.
 
         Args:
-            samples (torch.Tensor): Samples to be energy-freed.
+            samples: Samples to be energy-freed.
 
         Returns:
-            The system's energy based on input samples.
+            (torch.Tensor): The system's energy based on input samples.
 
         """
 
@@ -412,14 +396,14 @@ class RBM(Model):
 
         return energy
 
-    def pseudo_likelihood(self, samples):
+    def pseudo_likelihood(self, samples: torch.Tensor) -> torch.Tensor:
         """Calculates the logarithm of the pseudo-likelihood.
 
         Args:
-            samples (torch.Tensor): Samples to be calculated.
+            samples: Samples to be calculated.
 
         Returns:
-            The logarithm of the pseudo-likelihood based on input samples.
+            (torch.Tensor): The logarithm of the pseudo-likelihood based on input samples.
 
         """
 
@@ -430,56 +414,61 @@ class RBM(Model):
         energy = self.energy(samples_binary)
 
         # Samples an array of indexes to flip the bits
-        indexes = torch.randint(0, self.n_visible, size=(
-            samples.size(0), 1), device=self.device)
+        indexes = torch.randint(
+            0, self.n_visible, size=(samples.size(0), 1), device=self.device
+        )
 
         # Creates an empty vector for filling the indexes
-        bits = torch.zeros(samples.size(
-            0), samples.size(1), device=self.device)
+        bits = torch.zeros(samples.size(0), samples.size(1), device=self.device)
 
         # Fills the sampled indexes with 1
         bits = bits.scatter_(1, indexes, 1)
 
         # Actually flips the bits
-        samples_binary = torch.where(
-            bits == 0, samples_binary, 1 - samples_binary)
+        samples_binary = torch.where(bits == 0, samples_binary, 1 - samples_binary)
 
         # Calculates the energy after flipping the bits
         energy1 = self.energy(samples_binary)
 
         # Calculate the logarithm of the pseudo-likelihood
-        pl = torch.mean(self.n_visible *
-                        torch.log(torch.sigmoid(energy1 - energy) + c.EPSILON))
+        pl = torch.mean(
+            self.n_visible * torch.log(torch.sigmoid(energy1 - energy) + c.EPSILON)
+        )
 
         return pl
 
-    def fit(self, dataset, batch_size=128, epochs=10):
+    def fit(
+        self,
+        dataset: torch.utils.data.Dataset,
+        batch_size: Optional[int] = 128,
+        epochs: Optional[int] = 10,
+    ) -> Tuple[float, float]:
         """Fits a new RBM model.
 
         Args:
-            dataset (torch.utils.data.Dataset): A Dataset object containing the training data.
-            batch_size (int): Amount of samples per batch.
-            epochs (int): Number of training epochs.
+            dataset: A Dataset object containing the training data.
+            batch_size: Amount of samples per batch.
+            epochs: Number of training epochs.
 
         Returns:
-            MSE (mean squared error) and log pseudo-likelihood from the training step.
+            (Tuple[float, float]): MSE (mean squared error) and log pseudo-likelihood from the training step.
 
         """
 
         # Transforming the dataset into training batches
-        batches = DataLoader(dataset, batch_size=batch_size,
-                             shuffle=True, num_workers=0)
+        batches = DataLoader(
+            dataset, batch_size=batch_size, shuffle=True, num_workers=0
+        )
 
         # For every epoch
         for epoch in range(epochs):
-            logger.info('Epoch %d/%d', epoch+1, epochs)
+            logger.info("Epoch %d/%d", epoch + 1, epochs)
 
             # Calculating the time of the epoch's starting
             start = time.time()
 
             # Resetting epoch's MSE and pseudo-likelihood to zero
-            mse = 0
-            pl = 0
+            mse, pl = 0, 0
 
             # For every batch
             for samples, _ in tqdm(batches):
@@ -487,7 +476,7 @@ class RBM(Model):
                 samples = samples.reshape(len(samples), self.n_visible)
 
                 # Checking whether GPU is avaliable and if it should be used
-                if self.device == 'cuda':
+                if self.device == "cuda":
                     # Applies the GPU usage to the data
                     samples = samples.cuda()
 
@@ -498,8 +487,9 @@ class RBM(Model):
                 visible_states = visible_states.detach()
 
                 # Calculates the loss for further gradients' computation
-                cost = torch.mean(self.energy(samples)) - \
-                    torch.mean(self.energy(visible_states))
+                cost = torch.mean(self.energy(samples)) - torch.mean(
+                    self.energy(visible_states)
+                )
 
                 # Initializing the gradient
                 self.optimizer.zero_grad()
@@ -515,7 +505,8 @@ class RBM(Model):
 
                 # Calculating current's batch MSE
                 batch_mse = torch.div(
-                    torch.sum(torch.pow(samples - visible_states, 2)), batch_size).detach()
+                    torch.sum(torch.pow(samples - visible_states, 2)), batch_size
+                ).detach()
 
                 # Calculating the current's batch logarithm pseudo-likelihood
                 batch_pl = self.pseudo_likelihood(samples).detach()
@@ -532,24 +523,26 @@ class RBM(Model):
             end = time.time()
 
             # Dumps the desired variables to the model's history
-            self.dump(mse=mse.item(), pl=pl.item(), time=end-start)
+            self.dump(mse=mse.item(), pl=pl.item(), time=end - start)
 
-            logger.info('MSE: %f | log-PL: %f', mse, pl)
+            logger.info("MSE: %f | log-PL: %f", mse, pl)
 
         return mse, pl
 
-    def reconstruct(self, dataset):
+    def reconstruct(
+        self, dataset: torch.utils.data.Dataset
+    ) -> Tuple[float, torch.Tensor]:
         """Reconstructs batches of new samples.
 
         Args:
-            dataset (torch.utils.data.Dataset): A Dataset object containing the testing data.
+            dataset: A Dataset object containing the testing data.
 
         Returns:
-            Reconstruction error and visible probabilities, i.e., P(v|h).
+            (Tuple[float, torch.Tensor]): Reconstruction error and visible probabilities, i.e., P(v|h).
 
         """
 
-        logger.info('Reconstructing new samples ...')
+        logger.info("Reconstructing new samples ...")
 
         # Resetting MSE to zero
         mse = 0
@@ -558,8 +551,9 @@ class RBM(Model):
         batch_size = len(dataset)
 
         # Transforming the dataset into training batches
-        batches = DataLoader(dataset, batch_size=batch_size,
-                             shuffle=False, num_workers=0)
+        batches = DataLoader(
+            dataset, batch_size=batch_size, shuffle=False, num_workers=0
+        )
 
         # For every batch
         for samples, _ in tqdm(batches):
@@ -567,7 +561,7 @@ class RBM(Model):
             samples = samples.reshape(len(samples), self.n_visible)
 
             # Checking whether GPU is avaliable and if it should be used
-            if self.device == 'cuda':
+            if self.device == "cuda":
                 # Applies the GPU usage to the data
                 samples = samples.cuda()
 
@@ -575,12 +569,12 @@ class RBM(Model):
             _, pos_hidden_states = self.hidden_sampling(samples)
 
             # Calculating visible probabilities and states
-            visible_probs, visible_states = self.visible_sampling(
-                pos_hidden_states)
+            visible_probs, visible_states = self.visible_sampling(pos_hidden_states)
 
             # Calculating current's batch reconstruction MSE
             batch_mse = torch.div(
-                torch.sum(torch.pow(samples - visible_states, 2)), batch_size)
+                torch.sum(torch.pow(samples - visible_states, 2)), batch_size
+            )
 
             # Summing up the reconstruction's MSE
             mse += batch_mse
@@ -588,18 +582,18 @@ class RBM(Model):
         # Normalizing the MSE with the number of batches
         mse /= len(batches)
 
-        logger.info('MSE: %f', mse)
+        logger.info("MSE: %f", mse)
 
         return mse, visible_probs
 
-    def forward(self, x):
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
         """Performs a forward pass over the data.
 
         Args:
-            x (torch.Tensor): An input tensor for computing the forward pass.
+            x: An input tensor for computing the forward pass.
 
         Returns:
-            A tensor containing the RBM's outputs.
+            (torch.Tensor): A tensor containing the RBM's outputs.
 
         """
 
