@@ -2,6 +2,7 @@
 """
 
 import time
+from typing import Optional, Tuple
 
 import torch
 import torch.nn.functional as F
@@ -10,10 +11,10 @@ from tqdm import tqdm
 
 import learnergy.utils.constants as c
 import learnergy.utils.exception as e
-import learnergy.utils.logging as l
 from learnergy.models.bernoulli import ConvRBM
+from learnergy.utils import logging
 
-logger = l.get_logger(__name__)
+logger = logging.get_logger(__name__)
 
 
 class GaussianConvRBM(ConvRBM):
@@ -27,57 +28,70 @@ class GaussianConvRBM(ConvRBM):
 
     """
 
-    def __init__(self, visible_shape=(28, 28), filter_shape=(7, 7), n_filters=5, n_channels=1,
-                 steps=1, learning_rate=0.1, momentum=0, decay=0, use_gpu=False):
+    def __init__(
+        self,
+        visible_shape: Optional[Tuple[int, int]] = (28, 28),
+        filter_shape: Optional[Tuple[int, int]] = (7, 7),
+        n_filters: Optional[int] = 5,
+        n_channels: Optional[int] = 1,
+        steps: Optional[int] = 1,
+        learning_rate: Optional[float] = 0.1,
+        momentum: Optional[float] = 0.0,
+        decay: Optional[float] = 0.0,
+        use_gpu: Optional[bool] = False,
+    ) -> None:
         """Initialization method.
 
         Args:
-            visible_shape (tuple): Shape of visible units.
-            filter_shape (tuple): Shape of filters.
-            n_filters (int): Number of filters.
-            n_channels (int): Number of channels.
-            steps (int): Number of Gibbs' sampling steps.
-            learning_rate (float): Learning rate.
-            momentum (float): Momentum parameter.
-            decay (float): Weight decay used for penalization.
-            use_gpu (boolean): Whether GPU should be used or not.
+            visible_shape: Shape of visible units.
+            filter_shape: Shape of filters.
+            n_filters: Number of filters.
+            n_channels: Number of channels.
+            steps: Number of Gibbs' sampling steps.
+            learning_rate: Learning rate.
+            momentum: Momentum parameter.
+            decay: Weight decay used for penalization.
+            use_gpu: Whether GPU should be used or not.
 
         """
 
+        logger.info("Overriding class: ConvRBM -> GaussianConvRBM.")
+
+        super(GaussianConvRBM, self).__init__(
+            visible_shape,
+            filter_shape,
+            n_filters,
+            n_channels,
+            steps,
+            learning_rate,
+            momentum,
+            decay,
+            use_gpu,
+        )
+
         # Inner data normalization
         self.normalize = True
 
-        logger.info('Overriding class: ConvRBM -> GaussianConvRBM.')
-
-        # Override its parent class
-        super(GaussianConvRBM, self).__init__(visible_shape, filter_shape, n_filters, n_channels,
-                                              steps, learning_rate, momentum, decay, use_gpu)
-
-        logger.info('Class overrided.')
+        logger.info("Class overrided.")
 
     @property
-    def normalize(self):
-        """bool: Inner data normalization.
-
-        """
+    def normalize(self) -> bool:
+        """Inner data normalization."""
 
         return self._normalize
 
     @normalize.setter
-    def normalize(self, normalize):
-        if not isinstance(normalize, bool):
-            raise e.TypeError('`normalize` should be a boolean')
-
+    def normalize(self, normalize: bool) -> None:
         self._normalize = normalize
 
-    def hidden_sampling(self, v):
+    def hidden_sampling(self, v: torch.Tensor) -> Tuple[torch.Tensor, torch.Tensor]:
         """Performs the hidden layer sampling, i.e., P(h|v).
 
         Args:
             v (torch.Tensor): A tensor incoming from the visible layer.
 
         Returns:
-            The probabilities and states of the hidden layer sampling.
+            (Tuple[torch.Tensor, torch.Tensor]): The probabilities and states of the hidden layer sampling.
 
         """
         # Calculating neurons' activations
@@ -88,14 +102,14 @@ class GaussianConvRBM(ConvRBM):
 
         return probs, probs
 
-    def visible_sampling(self, h):
+    def visible_sampling(self, h: torch.Tensor) -> Tuple[torch.Tensor, torch.Tensor]:
         """Performs the visible layer sampling, i.e., P(v|h).
 
         Args:
-            h (torch.Tensor): A tensor incoming from the hidden layer.
+            h: A tensor incoming from the hidden layer.
 
         Returns:
-            The probabilities and states of the visible layer sampling.
+            (Tuple[torch.Tensor, torch.Tensor]): The probabilities and states of the visible layer sampling.
 
         """
 
@@ -114,26 +128,32 @@ class GaussianConvRBM(ConvRBM):
 
         return probs, probs
 
-    def fit(self, dataset, batch_size=128, epochs=10):
-        """Fits a new RBM model.
+    def fit(
+        self,
+        dataset: torch.utils.data.Dataset,
+        batch_size: Optional[int] = 128,
+        epochs: Optional[int] = 10,
+    ) -> float:
+        """Fits a new GaussianConvRBM model.
 
         Args:
-            dataset (torch.utils.data.Dataset): A Dataset object containing the training data.
-            batch_size (int): Amount of samples per batch.
-            epochs (int): Number of training epochs.
+            dataset: A Dataset object containing the training data.
+            batch_size: Amount of samples per batch.
+            epochs: Number of training epochs.
 
         Returns:
-            MSE (mean squared error) from the training step.
+            (float): MSE (mean squared error) from the training step.
 
         """
 
         # Transforming the dataset into training batches
-        batches = DataLoader(dataset, batch_size=batch_size,
-                             shuffle=True, num_workers=0)
+        batches = DataLoader(
+            dataset, batch_size=batch_size, shuffle=True, num_workers=0
+        )
 
         # For every epoch
         for epoch in range(epochs):
-            logger.info('Epoch %d/%d', epoch+1, epochs)
+            logger.info("Epoch %d/%d", epoch + 1, epochs)
 
             # Calculating the time of the epoch's starting
             start = time.time()
@@ -144,18 +164,24 @@ class GaussianConvRBM(ConvRBM):
             # For every batch
             for samples, _ in tqdm(batches):
                 # Guarantee the samples' batch
-                samples = samples.reshape(len(samples), self.n_channels, self.visible_shape[0], self.visible_shape[1])
+                samples = samples.reshape(
+                    len(samples),
+                    self.n_channels,
+                    self.visible_shape[0],
+                    self.visible_shape[1],
+                )
 
                 # Checking whether GPU is avaliable and if it should be used
-                if self.device == 'cuda':
+                if self.device == "cuda":
                     # Applies the GPU usage to the data
                     samples = samples.cuda()
 
                 # If it is supposed to use normalization
                 if self.normalize:
                     # Performs the normalization
-                    samples = ((samples - torch.mean(samples, 0, True)) /
-                               (torch.std(samples, 0, True) + c.EPSILON))
+                    samples = (samples - torch.mean(samples, 0, True)) / (
+                        torch.std(samples, 0, True) + c.EPSILON
+                    )
 
                 # Performs the Gibbs sampling procedure
                 _, _, _, _, visible_states = self.gibbs_sampling(samples)
@@ -164,8 +190,9 @@ class GaussianConvRBM(ConvRBM):
                 visible_states = visible_states.detach()
 
                 # Calculates the loss for further gradients' computation
-                cost = torch.mean(self.energy(samples)) - \
-                    torch.mean(self.energy(visible_states))
+                cost = torch.mean(self.energy(samples)) - torch.mean(
+                    self.energy(visible_states)
+                )
 
                 # Initializing the gradient
                 self.optimizer.zero_grad()
@@ -178,7 +205,8 @@ class GaussianConvRBM(ConvRBM):
 
                 # Calculating current's batch MSE
                 batch_mse = torch.div(
-                    torch.sum(torch.pow(samples - visible_states, 2)), batch_size).detach()
+                    torch.sum(torch.pow(samples - visible_states, 2)), batch_size
+                ).detach()
 
                 # Summing up to epochs' MSE
                 mse += batch_mse
@@ -190,8 +218,8 @@ class GaussianConvRBM(ConvRBM):
             end = time.time()
 
             # Dumps the desired variables to the model's history
-            self.dump(mse=mse.item(), time=end-start)
+            self.dump(mse=mse.item(), time=end - start)
 
-            logger.info('MSE: %f', mse)
+            logger.info("MSE: %f", mse)
 
         return mse
